package alarmclock;

import java.util.Properties;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

/**
 * Title:        828S Class alarmclock
 * Description:  Properties file
 * Copyright:    Copyright (c) 2002
 * Company:      UMD - Computer Science
 * @author  Daniel Camara
 * @version 1.0
 */

public class AssignmentProperties {



  /**
   * The static reference for the  properties
   */
  private static AssignmentProperties properties;

  public static final String defaultPropertieFile = "alarmclock.properties";

  /**
   * The properties file that contains the properties of this sistem
   */
   private Properties propertieEntries;

  /**
   * The properties file that contains the properties of  this sistem
   */
   public String propertiesFile = defaultPropertieFile;

   public String alarmSunday = "10:10";
   public boolean sundayCheckBox = false;
   public String alarmMonday = "10:10";
   public boolean mondayCheckBox = false;
   public String alarmTuesday = "10:10";
   public boolean tuesdayCheckBox = false;
   public String alarmWednesday = "10:10";
   public boolean wednesdayCheckBox = false;
   public String alarmThusday = "10:10";
   public boolean thusdayCheckBox = false;
   public String alarmFriday = "10:10";
   public boolean fridayCheckBox = false;
   public String alarmSaturday = "10:10";
   public boolean saturdayCheckBox = false;


  /**
   * Properties file constructor
   */
  private AssignmentProperties() {
   try{
        FileInputStream propertieFile = new FileInputStream(defaultPropertieFile);
        Properties propertieEntries = new Properties();
        propertieEntries.load(propertieFile);

        //If the properties file is diferent of the default
        String tempPropertiesFile = propertieEntries.getProperty("propertiesFile", propertiesFile);
        if(!propertiesFile.equals(tempPropertiesFile)) {
          propertieFile = new FileInputStream(propertiesFile);
          propertieEntries.load(propertieFile);
          propertiesFile = propertieEntries.getProperty("PropertiesFile", tempPropertiesFile);
        }
    }catch(FileNotFoundException e){
      System.out.println("Properties file (properties.property) not found.");
    }catch(IOException e){
      System.out.println("Properties file (properties.property) not found.");
    }
    // Puts the values at the real cells
   }

  /**
   * Generates a Properties object
   *
   * @return Properties - The default propertie
   */
  public static AssignmentProperties getInstance(){

    if(properties == null) {
      properties = new AssignmentProperties();
      properties = (AssignmentProperties)HandleProperties.loadParameters(properties.propertiesFile,
                                                               properties);
    }

    return properties;
  }

  /**
   * Save the properties to the propertiesFile file.  If you want to change
   * the file you need to change this variable.  It also changes the properties
   * at the default project properties.
   */
  public void saveProperties(){

    // if the actual propertie file is different than the main properties file,
    // updates the default file to reflect the changes
    if (properties.propertiesFile != defaultPropertieFile){
       try{
            //loads the main properties file
            FileInputStream propertieInputFile = new FileInputStream(defaultPropertieFile);
            Properties propertieEntries = new Properties();
            propertieEntries.load(propertieInputFile);
            propertieEntries.setProperty("propertiesFile", properties.propertiesFile);
            propertieInputFile.close();

            // stores the properties file name at the main properties file
            FileOutputStream propertiesOutputFile = new FileOutputStream(defaultPropertieFile);
            propertieEntries.store(propertiesOutputFile,"Last Alteration");
            propertiesOutputFile.flush();
            propertiesOutputFile.close();
        }catch(FileNotFoundException e){
          System.out.println("Properties file (properties.property) not found.");
        }catch(IOException e){
          System.out.println("Properties file (properties.property) not found.");
        }
      }

      HandleProperties.saveProperties(properties.propertiesFile, properties);
  }

  /**
   * Main function, used only for test purposes
   */
  public static void main(String[] args) {
   AssignmentProperties properties = AssignmentProperties.getInstance();
   properties.saveProperties();
  }

}

