package alarmclock;

import java.util.Properties;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.Vector;

/**
 * Title:
 *
 * Description:  Handle properties files and objects. From an object
 * finds a the propertie file the entryes that corresponds to the object field.
 * The actual value of the objects fields are used as default values.
 *
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Daniel Camara
 * @version 1.0
 */

public class HandleProperties {


  /**
   * loads the default propertiesInstance from the properties file
   *
   * @param propertiesFile - File that will be used as font to get
   * properties values.
   *
   * @param propertiesInstance - the object that will hold the new parameters
   */
  public static Object loadParameters(String propertiesFile, Object propertiesInstance){

     Properties properties = getProperties(propertiesFile);

     if(properties!=null){

         //Grabs the Fields of the object passed as parameter
         Field[] propertiesFields = propertiesInstance.getClass().getDeclaredFields();

        try{
            // to each field of the properties object try to find a correspondent at the
            // properties file
            for(int i = 0; i < propertiesFields.length; i++){

              int modifier = propertiesFields[i].getModifiers();

              // if may I change the field value directly
              if((Modifier.isPublic(modifier)) && !(Modifier.isFinal(modifier))) {

                // if the object field is a String
                if(propertiesFields[i].getType().getName().equals("java.lang.String")) {

                   // gets the property of the file that have the same name of the
                   // evaluated field of the object.
                   String parameterTmp = properties.getProperty(propertiesFields[i].getName(),
                                                     ((String)propertiesFields[i].get(propertiesInstance)));

                    // returns the right value to the object
                     propertiesFields[i].set(propertiesInstance, parameterTmp);
                } else
                  if(propertiesFields[i].getType().getName().equals("int")) {

                    try {
                       int parameterTmp = Integer.parseInt(properties.getProperty(propertiesFields[i].getName(),
                                                          (propertiesFields[i].getInt(propertiesInstance)+"")));
                       propertiesFields[i].setInt(propertiesInstance,parameterTmp);
                    }catch(Exception e){
                       e.printStackTrace();
                    }
                  } else
                  if(propertiesFields[i].getType().getName().equals("boolean")) {

                    String booleanValue = properties.getProperty(propertiesFields[i].getName());

                    //If the parameter is defined, if not permaneces the default
                    //stated at the variable creation
                    if(booleanValue != null)
                      if (booleanValue.equals("true")){
                        propertiesFields[i].setBoolean(propertiesInstance,true);
                      } else
                        if (booleanValue.equals("false")){
                          propertiesFields[i].setBoolean(propertiesInstance,false);
                        }


                  } else
                  if(propertiesFields[i].getType().getName().equals("float")) {

                    try {
                       float parameterTmp = Float.parseFloat(properties.getProperty(propertiesFields[i].getName(),
                                                          (propertiesFields[i].getFloat(propertiesInstance) + "")));
                       propertiesFields[i].setFloat(propertiesInstance, parameterTmp);
                    }catch(Exception e){
                       e.printStackTrace();
                    }

                  } else
                  if(propertiesFields[i].getType().getName().equals("double")) {
                    try {
                       double parameterTmp = Double.parseDouble(properties.getProperty(propertiesFields[i].getName(),
                                                          (propertiesFields[i].getDouble(propertiesInstance) + "")));
                       propertiesFields[i].setDouble(propertiesInstance, parameterTmp);
                    }catch(Exception e){
                       e.printStackTrace();
                    }
                  } else
                    if(propertiesFields[i].getType().getName().equals("char")) {
                      try {
                          String parameterTmp = properties.getProperty(propertiesFields[i].getName(),
                                                            (propertiesFields[i].getChar(propertiesInstance) + ""));
                         propertiesFields[i].setChar(propertiesInstance, parameterTmp.charAt(0));
                      }catch(Exception e){
                         e.printStackTrace();
                      }
                    } else
                      if(propertiesFields[i].getType().getName().equals("long")) {
                        try {
                           long parameterTmp = Long.parseLong(properties.getProperty(propertiesFields[i].getName(),
                                                              (propertiesFields[i].getLong(propertiesInstance) + "")));
                           propertiesFields[i].setLong(propertiesInstance, parameterTmp);
                        }catch(Exception e){
                           e.printStackTrace();
                        }
                      } else
                          if(propertiesFields[i].getType().getName().equals("byte")) {
                            try {
                               byte parameterTmp = Byte.parseByte(properties.getProperty(propertiesFields[i].getName(),
                                                                  (propertiesFields[i].getByte(propertiesInstance) + "")));
                               propertiesFields[i].setByte(propertiesInstance, parameterTmp);
                            }catch(Exception e){
                               e.printStackTrace();
                            }
                          }
/*                           else
                           if(propertiesFields[i].getType().isInstance(new Vector())) {
                                 Vector parameterTmp = (Vector)properties.get(propertiesFields[i].getName());

                                // returns the right value to the object
                                 propertiesFields[i].set(propertiesInstance, parameterTmp);
                }
*/

            }// if !(final)
          } //for
        }catch(Exception e){
          e.printStackTrace();
        }
    }//if properties != null
    return propertiesInstance;
  }

 /**
  * Saves the properties of the object to the properties file
  *
  * @param propertiesFile - The file properties that will store the values
  * @param propertiesValues - The object that contains the parameters to store
  *
  */
  public static void saveProperties(String propertiesFile, Object propertiesValues){

      Properties properties = new Properties();

      if(properties!=null){

         //Grabs the Fields of the object passed as parameter
         Field[] propertiesFields = propertiesValues.getClass().getDeclaredFields();

        try{
            // to each field of the properties object try to find a correspondent at the
            // properties file
            for(int i = 0; i < propertiesFields.length; i++){

              int modifier = propertiesFields[i].getModifiers();

              // if may I change the field value directly
              if((Modifier.isPublic(modifier)) && !(Modifier.isFinal(modifier))) {

                // if the object field is a String
                if(propertiesFields[i].getType().getName().equals("java.lang.String")) {

                   // stores the object at the properties
                   properties.setProperty(propertiesFields[i].getName(),
                                                ((String)propertiesFields[i].get(propertiesValues)));
                } else
                if(propertiesFields[i].getType().getName().equals("int")) {
                     properties.setProperty(propertiesFields[i].getName(),
                                              (propertiesFields[i].getInt(propertiesValues)+""));
                } else
                if(propertiesFields[i].getType().getName().equals("boolean")) {
                     properties.setProperty(propertiesFields[i].getName(),
                                              (propertiesFields[i].getBoolean(propertiesValues)+""));
                } else
                if(propertiesFields[i].getType().getName().equals("float")) {
                     properties.setProperty(propertiesFields[i].getName(),
                                              (propertiesFields[i].getFloat(propertiesValues)+""));
                } else
                if(propertiesFields[i].getType().getName().equals("double")) {
                     properties.setProperty(propertiesFields[i].getName(),
                                              (propertiesFields[i].getDouble(propertiesValues)+""));
                } else
                if(propertiesFields[i].getType().getName().equals("char")) {
                   properties.setProperty(propertiesFields[i].getName(),
                                            (propertiesFields[i].getChar(propertiesValues)+""));
                } else
                if(propertiesFields[i].getType().getName().equals("long")) {
                 properties.setProperty(propertiesFields[i].getName(),
                                          (propertiesFields[i].getLong(propertiesValues)+""));
                } else
                if(propertiesFields[i].getType().getName().equals("byte")) {
                  properties.setProperty(propertiesFields[i].getName(),
                                       (propertiesFields[i].getByte(propertiesValues)+""));
                }
/*                else
                if(propertiesFields[i].getType().isInstance(new Vector())) {
                     properties.setProperty(propertiesFields[i].getName(),
                                              (propertiesFields[i].get(propertiesValues)+""));
                }
*/
            }// if !(final)
          } //for

          // stores the values at the properties file
          FileOutputStream propertiesFileStream = new FileOutputStream(propertiesFile);
          properties.store(propertiesFileStream,"Last Alteration");
          propertiesFileStream.flush();
          propertiesFileStream.close();

        }catch(Exception e){
          e.printStackTrace();
        }
     }//if properties != null
   }


  /**
   * Creates a property object that reflects the gived properties file
   *
   * @param propertiesFile - The file that will be used as source of parameters
   *
   * @return Properties -  The properties of the gived file
   */
   private static Properties getProperties(String propertiesFile) {

   Properties properties = null;

   try{
        FileInputStream file = new FileInputStream(propertiesFile);
        properties = new Properties();
        properties.load(file);
    }catch(FileNotFoundException e){
      System.out.println("Erro : Parameters file (parametersT.property) not found.");
    }catch(IOException e){
      System.out.println("Erro : Parameters file (parametersT.property) not found.");
    }

    return properties;
  }

}
